#!/usr/bin/tclsh
# ABOUT
# This script periodicaly runs  'sensors' utility from lm_sensors 
# package, looks for specified parameters, checks whether they cross
# some predefined limits, and if they do - raises apropriate flag.
# The results writen to ~/.sensors  where they read by fmonitor.so plugin
# The format of ~/.sensors  is:
# name1:value1:flag1
# nameN:valueN:flagN

# CUSTOMIZATION
# 1. You can specify which paramaters, monitored by lm_sensors, you want
#    to see in gkrellm.
#    Edit sname and gname variables.
# 2. What are their limits. This controled by minlimits1, minlimits2,
#    maxlimits1, maxlimits2 valiables. If value is within limit1 -
#    no flag will be raised, falling between limit1 and limit2 - 
#    results in WARNING flag, and falling beyoind limit2 - results in
#    ALERT flag.
#    NOTE: if lm_sensors works correctly with your sensor chip, just copy 
#    min and max fields from 'sensors' output. And if not - put some 
#    reasonable values.
#
# EXAMPLE
# This is output of sensors utility  on my system. 
#  via686a-isa-e200
#  Adapter: ISA adapter
#  Algorithm: ISA algorithm
#  VCORE:     +1.85 V  (min =  +2.90 V, max =  +3.53 V)   
#  3.3V:      +3.32 V  (min =  +2.95 V, max =  +3.62 V)   
#  5.0V:      +5.68 V  (min =  +4.47 V, max =  +5.49 V)   
#  12V:      +12.28 V  (min = +26.65 V, max = +37.74 V)   
#  CPU Fan:  4500 RPM  (min =    0 RPM, div = 2)          
#  P/S Fan:     0 RPM  (min =    0 RPM, div = 2)          
#  CPU Temp: +42.8C   (limit = +40C, hysteresis = +45C) 
#  SBr Temp: +30.7C   (limit = +55C, hysteresis = +60C) 

# Say, I am interesting in CPU temp, system board temp, CPU fan speed and 
# vcore and also expect those values to not exceed some limits. 
# Here is (almost :-) correct settings to do this

## snames: parameter names in 'sensors' output 
set snames { "CPU Temp:" "SBr Temp:" "CPU Fan:" "VCORE:" }
## gnames: parameter names in gkrellm plugin
set gnames { "CPU" "SBr" "Fan1" "Vcore" }

#maxlimits1: if value crosses limits1 - WARNING flag is raised
set maxlimits1 { 45 35 10000 2.2} 
set minlimits1 { 20 20 1000  1.3}
#maxlimits2: if value crosses limits2 - ALERT flag is raised
set maxlimits2 { 50 38 10000 2.3} 
set minlimits2 { 20 20 1000  1.3}


# YOU NEED NOT CHANGE ANYTHING AFTER THIS LINE
set fname $env(HOME)/.sensors

while { 1 } {

    set iid [open "|sensors" "r"]

    while { ![eof $iid] } {
        set s [gets $iid]
	set name  "\[^:\]*:"
        set value "(\\+|-)?\[\[:digit:\]\]+\\.?\[\[:digit:\]\]*"
        if { ! [regexp "^($name)\[\[:space:\]\]*($value)" $s dummy n v] } {
            continue
        }
        set i [lsearch -exact $snames $n]
        if { $i == -1 } {
            continue
        }
        
        set output($i,gname) [lindex $gnames $i]
        set output($i,value) [string trimleft $v "+"]
		set min1 [lindex $minlimits1 $i]
		set max1 [lindex $maxlimits1 $i]
		set min2 [lindex $minlimits2 $i]
		set max2 [lindex $maxlimits2 $i]
		
        if { ($v > $min1) && ($v < $max1) } {
            set output($i,alert) ""
        } else {
            if { ($v <= $min1) || ($v >= $max1) } {                 
                set output($i,alert) "WARNING"
            }
            
            if { ($v <= $min2) || ($v >= $max2) } {                 
                set output($i,alert) "ALERT"
            }
        }
        continue
    }
    close $iid

    set oid [open $fname "w"]
    set len [llength $gnames] 
    for {set i 0} { $i < $len } { incr i } {
        puts $oid "$output($i,gname):$output($i,value):$output($i,alert)"
    }
    foreach  o [array names output] {
        unset output($o)
    }
    close $oid
    after  5000
}


